function love.load()
    canvas = love.graphics.newCanvas(love.graphics.getWidth(), love.graphics.getHeight())

    love.graphics.setCanvas(canvas)
        love.graphics.clear()

        love.graphics.rectangle("fill", love.graphics.getWidth() / 2, love.graphics.getHeight() / 2, 1, 1)
        love.graphics.rectangle("fill", 100, 100, 1, 1)
        love.graphics.rectangle("fill", 700, 100, 1, 1)
        love.graphics.rectangle("fill", 700, 700, 1, 1)
        love.graphics.rectangle("fill", 100, 700, 1, 1)
    love.graphics.setCanvas()

    fractCode = [[
    uniform vec2 resolution = vec2(800, 800);
    uniform float frame = 0;

    vec4 effect(vec4 color, Image image, vec2 texCoords, vec2 scrCoords) {
      int activeNeighbours = 0;
      vec4 actPixel;

      for (float x = -1; x <= 1; x++) {
        actPixel = Texel(image, vec2(texCoords.x + x/resolution.x, texCoords.y));

        if (actPixel.a == 1) {
          activeNeighbours++;
        }
      }

      for (float y = -1; y <= 1; y++) {
        actPixel = Texel(image, vec2(texCoords.x, texCoords.y + y/resolution.y));

        if (actPixel.a == 1) {
          activeNeighbours++;
        }
      }

      vec4 finalColor;
      if (activeNeighbours == 1) {
        finalColor = vec4(frame / 500, texCoords.x, texCoords.y, 1);
      } else {
        finalColor = vec4(0);
      }

      return finalColor;
    }
]]

    fract = love.graphics.newShader(fractCode)
    fract:send("resolution", {love.graphics.getWidth(), love.graphics.getHeight()})

    frame = 0
    paused = false

    canvasCopy = love.graphics.newCanvas(canvas:getWidth(), canvas:getHeight())

    sleepTime = 0.01
    coolDown = 0
end

function love.update(dt)
    if not paused then
        fract:send("frame", frame)

        coolDown = coolDown + dt
        if coolDown >= sleepTime then
            love.graphics.setCanvas(canvasCopy)
                love.graphics.clear()
                love.graphics.draw(canvas)
            love.graphics.setCanvas()

            love.graphics.setCanvas(canvas)
                love.graphics.setShader(fract)
                love.graphics.draw(canvasCopy)
                love.graphics.setShader()
            love.graphics.setCanvas()

            frame = frame + 1
            coolDown = 0
        end
    end

    love.window.setTitle("Fractal | ".."FPS:"..love.timer.getFPS().." | ".."Frame:"..frame.." | ".."Deltatime:"..sleepTime)
end

function love.draw()
    love.graphics.draw(canvas)
end

function love.keyreleased(key)
    if key == "backspace" then
        canvas:release()
        canvas = love.graphics.newCanvas(love.graphics.getWidth(), love.graphics.getHeight())
    elseif key == "return" then
        frame = 0
    elseif key == "space" then
        paused = not paused
    end
end

function love.mousereleased(x, y, button, isTouch)
    if button == 1 then
        love.graphics.setCanvas(canvas)
            love.graphics.rectangle("fill", x, y, 1, 1)
        love.graphics.setCanvas()
    end
end

function love.wheelmoved(x, y)
    if sleepTime - y / 100 >= 0 then
        sleepTime = sleepTime - y / 100
    end
end

function loadShader(fileName)
    local file = io.open(fileName)
    local shaderCode = file:read("*all")
    file:close()

    return shaderCode
end
